// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs

#include <sourcemod>
#include <sdktools>
#include <cstrike>

#pragma semicolon 1

#define PLUGIN_VERSION            "1.0"
#define PLUGIN_VERSION_CVAR       "forlix_sostats_version"

#define PLUGIN_COMMAND            "sostats"
#define PLUGIN_COMMAND_DEADTIME   0.75

#define PANEL_DISPLAYTIME         75

#define POINTS_INITIAL            1000
#define POINTS_MINIMUM            500
#define POINTS_DECAYTO            POINTS_INITIAL

#define TOPMENU_RANKS_PER_PAGE    10

#define STEAMID_PREFIX            "STEAM_0:"
#define STEAMID_FULL_LEN          32
#define STEAMID_PREFIX_LEN        8
#define STEAMID_CORE_LEN          STEAMID_FULL_LEN-STEAMID_PREFIX_LEN

#define PANEL_MAX_LENGTH          511
#define MAX_MSG_LEN               128

#define NAME_STR_EMPTY            "empty"
#define MENU_CONFIRM_SOUND        "buttons/button14.wav"

#define ANNOUNCEMENT_PREFIX       "\x04Forlix SoStats\x01 -"
#define ANNOUNCEMENT_PREFIX_PLAIN "Forlix SoStats"

#define NOT_SOCCER_MAP_MSG        "Forlix SoStats unloads (not a known soccer map)"

#define TRANSLATION_FILE          "forlix_sostats.phrases.txt"

public Plugin:myinfo =
{
  name = "Forlix SoStats",
  author = "Forlix (Dominik Friedrichs)",
  description = "Soccer stats and ranking for Forlix SoScore",
  version = PLUGIN_VERSION,
  url = "http://forlix.org/"
};

#define GOALS                     0
#define ASSISTS                   1
#define OWN_GOALS                 2
#define MISSES                    3
#define GROUND_PASSES             4
#define AIR_PASSES                5
#define STEALS                    6
#define LOSSES                    7
#define KEEPER_SAVES              8
#define KEEPER_FAILS              9
#define ROUNDS_WON                10
#define ROUNDS_LOST               11

#define PROPS_MAX                 11

new String:prop_desc[PROPS_MAX+1][16] =
{
  "GOAL",
  "ASSIST",
  "OWN_GOAL",
  "MISS",
  "GROUND_PASS",
  "AIR_PASS",
  "STEAL",
  "LOSS",
  "KEEPER_SAVE",
  "KEEPER_FAIL",
  "ROUND_WON",
  "ROUND_LOST"
};

new Handle:h_statsdb = INVALID_HANDLE;

new announcement_type = 0;
new tracking_min_players = 0;
new time_played_for_dbstore = 0;
new points_decay_start = 0;
new points_decay_period = 0;
new min_passes_for_rank = 0;
new allow_player_stats_reset = 0;
new hide_bare_chat_triggers = 0;
new max_rankcmds_per_round = 0;
new String:help_cmd[32];

new bool:map_started = false;
new bool:tracking_allowed = false;

new bool:p_spawned[MAXPLAYERS+1];
new p_rankcmds[MAXPLAYERS+1];
new p_hints[MAXPLAYERS+1];

new bool:p_init[MAXPLAYERS+1];

new String:p_steamid[MAXPLAYERS+1][STEAMID_CORE_LEN];
new String:p_name[MAXPLAYERS+1][MAX_NAME_LENGTH];

new p_pt_abs[MAXPLAYERS+1];
new p_pt_onj[MAXPLAYERS+1];
new p_rk_onj[MAXPLAYERS+1];

new p_props_abs[MAXPLAYERS+1][PROPS_MAX+1];
new p_props_onj[MAXPLAYERS+1][PROPS_MAX+1];

new p_tp_onj[MAXPLAYERS+1];
new p_fl_abs[MAXPLAYERS+1];

new p_jointime[MAXPLAYERS+1];

new players_tracked = 0;
new rk_lowest = 0;

#include "_sql_defines.inc"

#include "fs_convars.sp"
#include "fs_events.sp"
#include "fs_rank.sp"
#include "fs_top.sp"
#include "fs_stats.sp"
#include "fs_session.sp"
#include "fs_tracking.sp"
#include "fs_hints.sp"
#include "fs_panel.sp"
#include "fs_sql_main.sp"
#include "fs_toolfuncs.sp"

#undef REQUIRE_PLUGIN
#include "../Forlix_SoScore/include/forlix_soscore.inc"
