// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs

// SQL stored values:
//
// table sostats:
//
// _st steam id (format 0:12345678)
// _nm name

// _pt points
// _gs goals
// _as assists
// _og own goals
// _ms misses
// _gp ground passes
// _ap air passes
// _sl steals
// _ls losses
// _ks keeper saves
// _kf keeper fails
// _rw rounds won
// _rl rounds lost

// _tp time played
// _lc last connected

// _dc point decay
// _fl player flags:

#define PLAYER_INGAME                 (1<<0)
#define PLAYER_TRACKMSGS              (1<<1)

// table soranks:
// _rk rank
// _st steam id (format 0:12345678)

// buffer size used when assembling any query
#define SQL_QUERY_MAXLEN              512

#define SQL_SYSTEM_QUERY_PRIORITY     DBPrio_Normal
#define SQL_PLAYER_INTERACT_PRIORITY  DBPrio_Low

#define SQL_SOSTATS_TABLE_CREATE      "\
CREATE TABLE IF NOT EXISTS sostats \
(\
_st VARCHAR(16) NOT NULL PRIMARY KEY, \
_nm VARCHAR(32) NOT NULL DEFAULT '', \
\
_pt INTEGER NOT NULL DEFAULT 0, \
_gs INTEGER NOT NULL DEFAULT 0, \
_as INTEGER NOT NULL DEFAULT 0, \
_og INTEGER NOT NULL DEFAULT 0, \
_ms INTEGER NOT NULL DEFAULT 0, \
_gp INTEGER NOT NULL DEFAULT 0, \
_ap INTEGER NOT NULL DEFAULT 0, \
_sl INTEGER NOT NULL DEFAULT 0, \
_ls INTEGER NOT NULL DEFAULT 0, \
_ks INTEGER NOT NULL DEFAULT 0, \
_kf INTEGER NOT NULL DEFAULT 0, \
_rw INTEGER NOT NULL DEFAULT 0, \
_rl INTEGER NOT NULL DEFAULT 0, \
\
_tp INTEGER NOT NULL DEFAULT 0, \
_lc INTEGER NOT NULL DEFAULT 0, \
\
_dc DECIMAL NOT NULL DEFAULT 0.0, \
_fl INTEGER NOT NULL DEFAULT 0 \
);"

#define SQL_SORANKS_TABLE_CREATE      "\
CREATE TABLE IF NOT EXISTS soranks \
(\
_rk INTEGER NOT NULL PRIMARY KEY, \
_st VARCHAR(16) NOT NULL DEFAULT ''\
);"

#define SQL_PLAYER_CREATE             "\
INSERT INTO sostats \
(\
_st, \
_nm, \
_pt, \
_lc, \
_fl\
) \
VALUES \
(\
'%s', \
'%s', \
%u, \
%u, \
%u\
);"

#define SQL_PLAYER_READ               "\
SELECT \
_pt, \
_gs, \
_as, \
_og, \
_ms, \
_gp, \
_ap, \
_sl, \
_ls, \
_ks, \
_kf, \
_rw, \
_rl, \
\
_tp, \
\
_dc, \
_fl, \
r._rk \
FROM sostats \
LEFT JOIN soranks r \
ON sostats._st = r._st \
WHERE sostats._st = '%s';"

#define SQL_PLAYER_WRITE              "\
UPDATE sostats SET \
_nm='%s', \
\
_pt=%u, \
_gs=%u, \
_as=%u, \
_og=%u, \
_ms=%u, \
_gp=%u, \
_ap=%u, \
_sl=%u, \
_ls=%u, \
_ks=%u, \
_kf=%u, \
_rw=%u, \
_rl=%u, \
\
_tp=%u, \
_lc=%u, \
\
_dc=0, \
_fl=%u \
WHERE _st='%s';"

#define SQL_DROP_DECAYED_PLAYERS      "\
DELETE FROM sostats \
WHERE %u-_lc > %u;"

#define SQL_CLEAR_PLAYER_FLAGS        "\
UPDATE sostats SET \
_fl = _fl & ~%u;"

#define SQL_CALCULATE_DECAYS          "\
UPDATE sostats SET \
_dc=CASE \
WHEN %u-_lc > %u \
THEN round((%u-_lc-%u)/%u.0, 5) \
ELSE 0 \
END;"

#define SQL_PLAYER_DROP               "\
DELETE FROM sostats \
WHERE _st='%s';"

#define SQL_RANK_RESET                "\
UPDATE soranks SET \
_st='' \
WHERE _st='%s';"

#define SQL_WIPE_RANKS                "\
DELETE FROM soranks;"

#define SQL_CALCULATE_RANKS           "\
INSERT INTO soranks (_st) \
SELECT _st FROM sostats \
WHERE _gp+_ap >= %u \
ORDER BY \
_dc*(%u-_pt)+_pt DESC, \
_tp DESC;"

#define SQL_RANK_COUNT_READ           "\
SELECT COUNT() FROM soranks;"

#define SQL_PLAYER_COUNT_READ         "\
SELECT COUNT() FROM sostats;"

#define SQL_PLAYER_RANK_READ          "\
SELECT \
_rk, \
_pt, \
_gs/CAST(_ms as FLOAT), \
_sl/CAST(_ls as FLOAT), \
_ks/CAST(_kf as FLOAT) \
FROM sostats \
LEFT JOIN soranks \
ON soranks._st=sostats._st \
WHERE sostats._st='%s';"

#define SQL_PLAYER_TOP_READ           "\
SELECT \
_rk, \
_nm, \
_pt, \
_gs/CAST(_ms as FLOAT), \
_sl/CAST(_ls as FLOAT), \
_ks/CAST(_kf as FLOAT), \
\
_dc, \
_fl \
FROM soranks \
LEFT JOIN sostats s \
ON soranks._st=s._st \
LIMIT %u OFFSET %u;"

#define SQL_PLAYER_STATS_BY_STEAM     "\
SELECT \
_rk, \
\
sostats._st, \
_nm, \
\
_pt, \
_gs, \
_as, \
_og, \
_ms, \
_gp, \
_ap, \
_sl, \
_ls, \
_ks, \
_kf, \
_rw, \
_rl, \
\
_tp, \
_lc, \
\
_dc ,\
_fl \
FROM sostats \
LEFT JOIN soranks \
ON soranks._st=sostats._st \
WHERE sostats._st='%s';"

#define SQL_PLAYER_STATS_BY_RANK      "\
SELECT \
_rk, \
\
sostats._st, \
_nm, \
\
_pt, \
_gs, \
_as, \
_og, \
_ms, \
_gp, \
_ap, \
_sl, \
_ls, \
_ks, \
_kf, \
_rw, \
_rl, \
\
_tp, \
_lc, \
\
_dc, \
_fl \
FROM soranks \
LEFT JOIN sostats \
ON sostats._st=soranks._st \
WHERE soranks._rk=%u;"

#define SQL_PLAYER_SESSION_READ       "\
SELECT \
_rk \
FROM soranks \
WHERE _st='%s';"
