// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs

#include "_main.inc"

#define HIDE_TOP                  (1<<0)
#define HIDE_RANK                 (1<<1)
#define HIDE_STATS                (1<<2)
#define HIDE_SESSION              (1<<3)

#define STR_TOP                   "top"
#define STR_RANK                  "rank"
#define STR_STATS                 "stats"
#define STR_SESSION               "session"

#define STR_TOP_LEN               3
#define STR_RANK_LEN              4
#define STR_STATS_LEN             5
#define STR_SESSION_LEN           7

static bool:late_load = false;


public APLRes:AskPluginLoad2(Handle:myself,
                             bool:late,
                             String:error[],
                             err_max)
{
  late_load = late;
  return(APLRes_Success);
}


public OnPluginStart()
{
  RegConsoleCmd(PLUGIN_COMMAND, HelpPanel, "Opens the SoStats Help panel.");

  RegConsoleCmd("say", Command_Say);
  RegConsoleCmd("say_team", Command_Say);

  HookEvent("round_start", Event_RoundStart);
  HookEvent("round_end", Event_RoundEnd);

  HookEvent("player_spawn", Event_PlayerSpawn);
  HookEvent("player_changename", Event_PlayerChangeName);
  HookEvent("player_team", Event_PlayerTeam);

  SetupConVars();

  decl String:sql_error[256];
  h_statsdb = SQLite_UseDatabase("forlix_sostats", sql_error, sizeof(sql_error));

  if(h_statsdb == INVALID_HANDLE)
  {
    SetFailState("Failed to connect to database (%s)", sql_error);
    return;
  }

  SQL_LockDatabase(h_statsdb);

  if(!SQL_FastQuery(h_statsdb, SQL_SOSTATS_TABLE_CREATE)
  || !SQL_FastQuery(h_statsdb, SQL_SORANKS_TABLE_CREATE))
  {
    SQL_UnlockDatabase(h_statsdb);

    SetFailState("Failed to initialize tables");
    return;
  }

  SQL_UnlockDatabase(h_statsdb);

  ClearPlayerFlags(PLAYER_INGAME);
  RefreshDBMaintainedData();

  return;
}


public OnPluginEnd()
{
  WriteAllPlayersToDatabase(h_statsdb, true);
  return;
}


public OnMapStart()
{
  map_started = true;
  LoadTranslations(TRANSLATION_FILE);

  if(late_load)
    LateLoad();

  late_load = false;
  return;
}


public ForlixSoScore_OnNoSoccerMap()
{
  UnloadWithMessage(NOT_SOCCER_MAP_MSG);
  return;
}


public OnMapEnd()
{
  map_started = false;
}


public OnClientConnected(client)
{
  p_spawned[client] = false;
  p_hints[client] = 0;

  return;
}


public OnClientAuthorized(client, const String:str_networkid[])
{
  if(IsClientConnected(client))
  // make sure client is not already disconnected again
    ReadPlayerFromDatabase(h_statsdb, client, str_networkid[8]);

  return;
}


public OnClientDisconnect(client)
{
  WritePlayerToDatabase(h_statsdb, client, true);
  return;
}


public Action:Command_Say(client, args)
{
  if(!client)
    return(Plugin_Continue);

  decl String:text[64];
  new hide = 0;

  GetCmdArgString(text, sizeof(text));

  StripQuotes(text);
  StringToLower(text);

  if(!strncmp(text, STR_TOP, STR_TOP_LEN))
  {
    if(!text[STR_TOP_LEN])
    // no offset specified
    {
      Top(client, 0);
      hide = (hide_bare_chat_triggers & HIDE_TOP);
    }
    else
    if(IsStringInt(text[STR_TOP_LEN]))
    // offset specified
    {
      Top(client, StringToInt(text[STR_TOP_LEN]));
      hide = (hide_bare_chat_triggers & HIDE_TOP);
    }
  }
  else
  if(!strcmp(text, STR_RANK))
  {
    Rank(client);
    hide = (hide_bare_chat_triggers & HIDE_RANK);
  }
  else
  if(!strncmp(text, STR_STATS, STR_STATS_LEN))
  {
    if(!text[STR_STATS_LEN]
    || !strcmp(text[STR_STATS_LEN], "me"))
    // display stats for player himself
    {
      Stats(client, 0);
      hide = (hide_bare_chat_triggers & HIDE_STATS);
    }
    else
    if(IsStringInt(text[STR_STATS_LEN]))
    // display stats of specified rank
    {
      Stats(client, StringToInt(text[STR_STATS_LEN]));
      hide = (hide_bare_chat_triggers & HIDE_STATS);
    }
  }
  else
  if(!strcmp(text, STR_SESSION))
  {
    Session(client);
    hide = (hide_bare_chat_triggers & HIDE_SESSION);
  }

  if(hide)
    return(Plugin_Handled);

  return(Plugin_Continue);
}


UnloadWithMessage(const String:message[])
{
  PrintToServer("%s", message);
  ServerCommand("sm plugins unload forlix_sostats");

  return;
}
