// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs

// convar defaults
#define ANNOUNCEMENT_TYPE         "0"
#define TRACKING_MIN_PLAYERS      "3"
#define TIME_PLAYED_FOR_DBSTORE   "150"
#define POINTS_DECAY_START        "2"
#define POINTS_DECAY_PERIOD       "28"
#define MIN_PASSES_FOR_RANK       "25"
#define ALLOW_PLAYER_STATS_RESET  "1"
#define HIDE_BARE_CHAT_TRIGGERS   "0"
#define MAX_RANKCMDS_PER_ROUND    "3"
#define CHAT_TRIGGER              "!"

static Handle:h_announcement_type = INVALID_HANDLE;
static Handle:h_tracking_min_players = INVALID_HANDLE;
static Handle:h_time_played_for_dbstore = INVALID_HANDLE;
static Handle:h_points_decay_start = INVALID_HANDLE;
static Handle:h_points_decay_period = INVALID_HANDLE;
static Handle:h_min_passes_for_rank = INVALID_HANDLE;
static Handle:h_allow_player_stats_reset = INVALID_HANDLE;
static Handle:h_hide_bare_chat_triggers = INVALID_HANDLE;
static Handle:h_max_rankcmds_per_round = INVALID_HANDLE;
static Handle:h_chat_trigger = INVALID_HANDLE;


SetupConVars()
{
  new Handle:version_cvar = CreateConVar(PLUGIN_VERSION_CVAR,
  PLUGIN_VERSION,
  "Forlix SoStats plugin version",
  FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_NOTIFY|FCVAR_PRINTABLEONLY);

  SetConVarString(version_cvar, PLUGIN_VERSION, false, false);

  h_announcement_type = CreateConVar("forlix_sostats_announcement_type",
  ANNOUNCEMENT_TYPE,
  "How often the plugin may announce itself to players: Every round-start (0), Once after connect (1)",
  0, true, 0.0, true, 1.0);

  h_tracking_min_players = CreateConVar("forlix_sostats_tracking_min_players",
  TRACKING_MIN_PLAYERS,
  "Minimum amount of players required per team for tracking to be enabled",
  0, true, 1.0, true, 64.0);

  h_time_played_for_dbstore = CreateConVar("forlix_sostats_time_played_for_db_store",
  TIME_PLAYED_FOR_DBSTORE,
  "Time in seconds a new player has to remain on the server to be retained in the database",
  0, true, 2.0, true, 1200.0);

  h_points_decay_start = CreateConVar("forlix_sostats_points_decay_start",
  POINTS_DECAY_START,
  "Time in days from last connect after which a players points decay will start",
  0, true, 1.0, true, 30.0);

  h_points_decay_period = CreateConVar("forlix_sostats_points_decay_period",
  POINTS_DECAY_PERIOD,
  "Points will decay within this many days and a player be deleted at the end of this period",
  0, true, 7.0, true, 365.0);

  h_min_passes_for_rank = CreateConVar("forlix_sostats_min_passes_for_rank",
  MIN_PASSES_FOR_RANK,
  "Minimum amount of passes (ground+air) required for a player to be ranked",
  0, true, 1.0, true, 500.0);

  h_allow_player_stats_reset = CreateConVar("forlix_sostats_allow_player_stats_reset",
  ALLOW_PLAYER_STATS_RESET,
  "Whether (1) or not (0) players can reset their stats from within the SoStats help panel",
  0, true, 0.0, true, 1.0);

  h_hide_bare_chat_triggers = CreateConVar("forlix_sostats_hide_bare_chat_triggers",
  HIDE_BARE_CHAT_TRIGGERS,
  "Which bare chat triggers to hide (add as desired: +1 top, +2 rank, +4 stats, +8 session)",
  0, true, 0.0, true, 15.0);

  h_max_rankcmds_per_round = CreateConVar("forlix_sostats_max_rankcmds_per_round",
  MAX_RANKCMDS_PER_ROUND,
  "How many times per round a player can print his/her rank to everyone (0 for infinite)",
  0, true, 0.0, true, 16.0);

  h_chat_trigger = CreateConVar("forlix_sostats_chat_trigger",
  CHAT_TRIGGER,
  "Set this to one of the ChatTrigger strings configured in SourceMod's core.cfg",
  FCVAR_PRINTABLEONLY);

  HookConVarChange(h_announcement_type, MyConVarChanged);
  HookConVarChange(h_tracking_min_players, MyConVarChanged);
  HookConVarChange(h_time_played_for_dbstore, MyConVarChanged);
  HookConVarChange(h_points_decay_start, MyConVarChanged);
  HookConVarChange(h_points_decay_period, MyConVarChanged);
  HookConVarChange(h_min_passes_for_rank, MyConVarChanged);
  HookConVarChange(h_allow_player_stats_reset, MyConVarChanged);
  HookConVarChange(h_hide_bare_chat_triggers, MyConVarChanged);
  HookConVarChange(h_max_rankcmds_per_round, MyConVarChanged);
  HookConVarChange(h_chat_trigger, MyConVarChanged);

  // manually trigger convar readout
  MyConVarChanged(INVALID_HANDLE, "0", "0");

  return;
}


public MyConVarChanged(Handle:convar,
                       const String:oldValue[],
                       const String:newValue[])
{
  announcement_type = GetConVarInt(h_announcement_type);
  tracking_min_players = GetConVarInt(h_tracking_min_players);
  time_played_for_dbstore = GetConVarInt(h_time_played_for_dbstore);
  points_decay_start = GetConVarInt(h_points_decay_start);
  points_decay_period = GetConVarInt(h_points_decay_period);
  min_passes_for_rank = GetConVarInt(h_min_passes_for_rank);
  allow_player_stats_reset = GetConVarInt(h_allow_player_stats_reset);
  hide_bare_chat_triggers = GetConVarInt(h_hide_bare_chat_triggers);
  max_rankcmds_per_round = GetConVarInt(h_max_rankcmds_per_round);

  GetConVarString(h_chat_trigger, help_cmd, sizeof(help_cmd));
  StrCat(help_cmd, sizeof(help_cmd), PLUGIN_COMMAND);

  CheckTrackingAllowed();
  return;
}
