// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs

#define SECONDS_PER_DAY           60*60*24


ReadAllPlayersFromDatabase(Handle:db)
{
  decl String:str_networkid[STEAMID_FULL_LEN];

  SQL_BeginTransaction(db);

  for(new client = 1; client <= MaxClients; client++)
  if(IsClientConnected(client)
  && GetClientAuthString(client, str_networkid, sizeof(str_networkid)))
    ReadPlayerFromDatabase(db, client, str_networkid[8]);
  else
    p_init[client] = false;

  SQL_CommitTransaction(db);
  return;
}


ReadPlayerFromDatabase(Handle:db,
                       client,
                       const String:steamid_core[])
{
  if(p_init[client]
  || IsFakeClient(client))
    return;

  decl String:query[SQL_QUERY_MAXLEN];

  GetClientName(client, p_name[client], sizeof(p_name[]));
  MakeStringPrintable(p_name[client], sizeof(p_name[]), NAME_STR_EMPTY);

  strcopy(p_steamid[client], sizeof(p_steamid[]), steamid_core);

  Format(query, sizeof(query), SQL_PLAYER_READ, steamid_core);
  SQL_TQuery(db, SQL_OnPlayerRead, query, GetClientUserId(client), SQL_SYSTEM_QUERY_PRIORITY);

  return;
}


public SQL_OnPlayerRead(Handle:owner,
                        Handle:hndl,
                        const String:error[],
                        any:userid)
{
  if(hndl == INVALID_HANDLE)
  {
    LogError("SQL_OnPlayerRead failure: %s", error);
    return;
  }

  new client = GetClientOfUserId(userid);

  if(!client)
    return;

  if(SQL_HasResultSet(hndl)
  && SQL_FetchRow(hndl))
  // player exists in db, fetch fields
  {
    p_pt_abs[client] = SQL_FetchInt(hndl, 0);

    p_props_abs[client][GOALS] = SQL_FetchInt(hndl, 1);
    p_props_abs[client][ASSISTS] = SQL_FetchInt(hndl, 2);
    p_props_abs[client][OWN_GOALS] = SQL_FetchInt(hndl, 3);
    p_props_abs[client][MISSES] = SQL_FetchInt(hndl, 4);
    p_props_abs[client][GROUND_PASSES] = SQL_FetchInt(hndl, 5);
    p_props_abs[client][AIR_PASSES] = SQL_FetchInt(hndl, 6);
    p_props_abs[client][STEALS] = SQL_FetchInt(hndl, 7);
    p_props_abs[client][LOSSES] = SQL_FetchInt(hndl, 8);
    p_props_abs[client][KEEPER_SAVES] = SQL_FetchInt(hndl, 9);
    p_props_abs[client][KEEPER_FAILS] = SQL_FetchInt(hndl, 10);
    p_props_abs[client][ROUNDS_WON] = SQL_FetchInt(hndl, 11);
    p_props_abs[client][ROUNDS_LOST] = SQL_FetchInt(hndl, 12);

    p_jointime[client] = GetTime();
    p_tp_onj[client] = SQL_FetchInt(hndl, 13);

    // apply decay factor
    p_pt_abs[client] = DecayPoints(p_pt_abs[client], SQL_FetchFloat(hndl, 14));

    p_fl_abs[client] = SQL_FetchInt(hndl, 15) | PLAYER_INGAME;
    p_rk_onj[client] = SQL_FetchInt(hndl, 16);

    p_pt_onj[client] = p_pt_abs[client];
    p_props_onj[client] = p_props_abs[client];

    p_init[client] = true;

    // write back to db (updated name, updated flags)
    WritePlayerToDatabase(owner, client, false);
  }
  else
  // player not found in db, create entry
    CreatePlayerInDatabase(owner, client);

  return;
}


CreatePlayerInDatabase(Handle:db, client)
{
  if(p_init[client]
  || IsFakeClient(client))
    return;

  decl String:query[SQL_QUERY_MAXLEN];

  // set all values to their defaults
  p_pt_abs[client] = (p_pt_onj[client] = POINTS_INITIAL);

  for(new i = 0; i <= PROPS_MAX; i++)
  {
    p_props_abs[client][i] = 0;
    p_props_onj[client][i] = 0;
  }

  p_jointime[client] = GetTime();
  p_tp_onj[client] = 0;

  // flags for new players
  p_fl_abs[client] = PLAYER_INGAME|PLAYER_TRACKMSGS;
  p_rk_onj[client] = 0;

  p_init[client] = true;

  decl String:name_esc[MAX_NAME_LENGTH*2];
  SQL_EscapeString(db, p_name[client], name_esc, sizeof(name_esc));

  Format(query, sizeof(query), SQL_PLAYER_CREATE,
  p_steamid[client],
  name_esc,
  p_pt_abs[client],
  p_jointime[client],
  p_fl_abs[client]);
  SQL_TQuery(db, SQL_OnPlayerCreated, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  return;
}


WritePlayerToDatabase(Handle:db,
                      client,
                      bool:session_end)
{
  if(!p_init[client]
  || IsFakeClient(client))
    return;

  if(session_end)
  // player disconnect or plugin shutdown
  {
    if(TotalTimePlayed(client) < time_played_for_dbstore)
    // players total time played is too low, forget about him
    {
      DropPlayerFromDatabase(db, client);
      return;
    }

    p_fl_abs[client] &= ~PLAYER_INGAME;
    p_init[client] = false;
  }

  // save player data to db
  decl String:query[SQL_QUERY_MAXLEN];

  decl String:name_esc[MAX_NAME_LENGTH*2];
  SQL_EscapeString(db, p_name[client], name_esc, sizeof(name_esc));

  Format(query, sizeof(query), SQL_PLAYER_WRITE,
  name_esc,
  p_pt_abs[client],

  p_props_abs[client][GOALS],
  p_props_abs[client][ASSISTS],
  p_props_abs[client][OWN_GOALS],
  p_props_abs[client][MISSES],
  p_props_abs[client][GROUND_PASSES],
  p_props_abs[client][AIR_PASSES],
  p_props_abs[client][STEALS],
  p_props_abs[client][LOSSES],
  p_props_abs[client][KEEPER_SAVES],
  p_props_abs[client][KEEPER_FAILS],
  p_props_abs[client][ROUNDS_WON],
  p_props_abs[client][ROUNDS_LOST],

  TotalTimePlayed(client),
  GetTime(),
  p_fl_abs[client],

  p_steamid[client]);
  SQL_TQuery(db, SQL_GenPlayerSaved, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  return;
}


WriteAllPlayersToDatabase(Handle:db, bool:session_end)
{
  SQL_BeginTransaction(db);

  for(new client = 1; client <= MaxClients; client++)
  if(IsClientConnected(client))
    WritePlayerToDatabase(db, client, session_end);

  SQL_CommitTransaction(db);
  return;
}


DropPlayerFromDatabase(Handle:db, client)
{
  if(!p_init[client]
  || IsFakeClient(client))
    return;

  decl String:query[SQL_QUERY_MAXLEN];
  p_init[client] = false;

  Format(query, sizeof(query), SQL_PLAYER_DROP, p_steamid[client]);
  SQL_TQuery(db, SQL_GenPlayerDropped, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  Format(query, sizeof(query), SQL_RANK_RESET, p_steamid[client]);
  SQL_TQuery(db, SQL_GenPlayerRankReset, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  return;
}


ClearPlayerFlags(flags)
{
  decl String:query[SQL_QUERY_MAXLEN];

  Format(query, sizeof(query), SQL_CLEAR_PLAYER_FLAGS, flags);
  SQL_TQuery(h_statsdb, SQL_GenPlayerFlagsCleared, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  return;
}


RefreshDBMaintainedData()
{
  SQL_BeginTransaction(h_statsdb);

  decl String:query[SQL_QUERY_MAXLEN];
  new time = GetTime();

  Format(query, sizeof(query), SQL_DROP_DECAYED_PLAYERS,
         time, points_decay_start*SECONDS_PER_DAY+points_decay_period*SECONDS_PER_DAY);
  SQL_TQuery(h_statsdb, SQL_GenDecayedPlayersDropped, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  Format(query, sizeof(query), SQL_CALCULATE_DECAYS,
         time, points_decay_start*SECONDS_PER_DAY,
         time, points_decay_start*SECONDS_PER_DAY, points_decay_period*SECONDS_PER_DAY);
  SQL_TQuery(h_statsdb, SQL_GenPointDecaysCalculated, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  SQL_TQuery(h_statsdb, SQL_GenRanksWiped, SQL_WIPE_RANKS, _, SQL_SYSTEM_QUERY_PRIORITY);

  Format(query, sizeof(query), SQL_CALCULATE_RANKS, min_passes_for_rank, POINTS_DECAYTO);
  SQL_TQuery(h_statsdb, SQL_GenRanksCalculated, query, _, SQL_SYSTEM_QUERY_PRIORITY);

  SQL_TQuery(h_statsdb, SQL_OnLowestRankRead, SQL_RANK_COUNT_READ, _, SQL_SYSTEM_QUERY_PRIORITY);
  SQL_TQuery(h_statsdb, SQL_OnPlayerCountRead, SQL_PLAYER_COUNT_READ, _, SQL_SYSTEM_QUERY_PRIORITY);

  SQL_CommitTransaction(h_statsdb);
  return;
}


public SQL_OnLowestRankRead(Handle:owner,
                            Handle:hndl,
                            const String:error[],
                            any:data)
{
  if(hndl == INVALID_HANDLE)
  {
    LogError("SQL_OnLowestRankRead failure: %s", error);
    return;
  }

  if(!SQL_HasResultSet(hndl))
  {
    LogError("SQL_OnLowestRankRead failure: No result");
    return;
  }

  if(!SQL_FetchRow(hndl))
    rk_lowest = 0;
  else
    rk_lowest = SQL_FetchInt(hndl, 0);

  return;
}


public SQL_OnPlayerCountRead(Handle:owner,
                             Handle:hndl,
                             const String:error[],
                             any:data)
{
  if(hndl == INVALID_HANDLE)
  {
    LogError("SQL_OnPlayerCountRead failure: %s", error);
    return;
  }

  if(!SQL_HasResultSet(hndl))
  {
    LogError("SQL_OnPlayerCountRead failure: No result");
    return;
  }

  if(!SQL_FetchRow(hndl))
    players_tracked = 0;
  else
    players_tracked = SQL_FetchInt(hndl, 0);

  return;
}


SQL_BeginTransaction(Handle:db)
{
  SQL_TQuery(db, SQL_GenTransactionReturn, "BEGIN;", SQL_SYSTEM_QUERY_PRIORITY);
  return;
}


SQL_CommitTransaction(Handle:db)
{
  SQL_TQuery(db, SQL_GenTransactionReturn, "COMMIT;", SQL_SYSTEM_QUERY_PRIORITY);
  return;
}


// Gen callbacks:
// All callbacks that dont do anything

public SQL_OnPlayerCreated(Handle:owner,
                           Handle:hndl,
                           const String:error[],
                           any:data)
{
  if(hndl == INVALID_HANDLE)
  {
    LogError("SQL_OnPlayerCreated failure: %s", error);
    return;
  }

  return;
}


public SQL_GenPlayerSaved(Handle:owner,
                          Handle:hndl,
                          const String:error[],
                          any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenPlayerSaved failure: %s", error);

  return;
}


public SQL_GenPlayerDropped(Handle:owner,
                            Handle:hndl,
                            const String:error[],
                            any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenPlayerDropped failure: %s", error);

  return;
}


public SQL_GenPlayerRankReset(Handle:owner,
                              Handle:hndl,
                              const String:error[],
                              any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenPlayerRankReset failure: %s", error);

  return;
}


public SQL_GenPlayerFlagsCleared(Handle:owner,
                                 Handle:hndl,
                                 const String:error[],
                                 any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenPlayerFlagsCleared: %s", error);

  return;
}


public SQL_GenDecayedPlayersDropped(Handle:owner,
                                    Handle:hndl,
                                    const String:error[],
                                    any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenDecayedPlayersDropped: %s", error);

  return;
}


public SQL_GenTransactionReturn(Handle:owner,
                                Handle:hndl,
                                const String:error[],
                                any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenTransactionReturn failure: %s", error);

  return;
}


public SQL_GenPointDecaysCalculated(Handle:owner,
                                    Handle:hndl,
                                    const String:error[],
                                    any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenPointDecaysCalculated: %s", error);

  return;
}


public SQL_GenRanksWiped(Handle:owner,
                         Handle:hndl,
                         const String:error[],
                         any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenRanksWiped failure: %s", error);

  return;
}


public SQL_GenRanksCalculated(Handle:owner,
                              Handle:hndl,
                              const String:error[],
                              any:data)
{
  if(hndl == INVALID_HANDLE)
    LogError("SQL_GenRanksCalculated failure: %s", error);

  return;
}
