// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs


Stats(client, rank)
{
  if(!p_init[client]
  || DeadTimeBlock(client))
    return;

  decl String:query[SQL_QUERY_MAXLEN];

  if(rank)
  {
    if(rank > rk_lowest)
      rank = rk_lowest;

    Format(query, sizeof(query), SQL_PLAYER_STATS_BY_RANK, rank);
    SQL_TQuery(h_statsdb, SQL_OnPlayerStatsRead, query, GetClientUserId(client), SQL_PLAYER_INTERACT_PRIORITY);
  }
  else
  {
    Format(query, sizeof(query), SQL_PLAYER_STATS_BY_STEAM, p_steamid[client]);
    SQL_TQuery(h_statsdb, SQL_OnPlayerStatsRead, query, GetClientUserId(client), SQL_PLAYER_INTERACT_PRIORITY);
  }

  return;
}


public SQL_OnPlayerStatsRead(Handle:owner,
                             Handle:hndl,
                             const String:error[],
                             any:userid)
{
  if(hndl == INVALID_HANDLE)
  {
    LogError("SQL_OnPlayerStatsRead failure: %s", error);
    return;
  }

  if(!SQL_HasResultSet(hndl))
  {
    LogError("SQL_OnPlayerStatsRead failure: No result");
    return;
  }

  new client = GetClientOfUserId(userid);

  if(!client)
    return;

  if(!SQL_FetchRow(hndl))
  {
    PrintToChat(client, "%t", "NO_PLAYERS_RANKED");
    return;
  }

  new bool:self = false;

  new rk = SQL_FetchInt(hndl, 0);

  decl String:st[STEAMID_CORE_LEN];
  decl String:nm[MAX_NAME_LENGTH];

  SQL_FetchString(hndl, 1, st, sizeof(st));
  SQL_FetchString(hndl, 2, nm, sizeof(nm));

  new pt = SQL_FetchInt(hndl, 3);

  new gs = SQL_FetchInt(hndl, 4);
  new as = SQL_FetchInt(hndl, 5);
  new og = SQL_FetchInt(hndl, 6);
  new ms = SQL_FetchInt(hndl, 7);
  new gp = SQL_FetchInt(hndl, 8);
  new ap = SQL_FetchInt(hndl, 9);
  new sl = SQL_FetchInt(hndl, 10);
  new ls = SQL_FetchInt(hndl, 11);
  new ks = SQL_FetchInt(hndl, 12);
  new kf = SQL_FetchInt(hndl, 13);
  new rw = SQL_FetchInt(hndl, 14);
  new rl = SQL_FetchInt(hndl, 15);

  new tp = SQL_FetchInt(hndl, 16);
  new lc = SQL_FetchInt(hndl, 17);

  // apply decay factor
  pt = DecayPoints(pt, SQL_FetchFloat(hndl, 18));

  new fl = SQL_FetchInt(hndl, 19);

  if(!nm[0])
  {
    PrintToChat(client, "%t.", "PLAYERS_RANK_RESET");
    return;
  }

  if(!strcmp(st, p_steamid[client]))
  // fetched stats are for client himself
    self = true;

  new Handle:pn = CreatePanel();
  SetGlobalTransTarget(client);

  decl String:buf[256];

  if(self)
    Format(buf, sizeof(buf), "Forlix SoStats                StatsMe\n ");
  else
    Format(buf, sizeof(buf), "Forlix SoStats                Stats%u\n ", rk);

  DrawPanelText(pn, buf);

  decl String:name[MAX_NAME_LENGTH];

  new len = strcopy(name, sizeof(name), nm);
  TruncateString(name, len, 20);
  Format(buf, sizeof(buf), "%t %s", "PLAYER", name);
  DrawPanelText(pn, buf);

  if(rk)
    Format(buf, sizeof(buf), "%t %u   %t %u\n ", "POINTS", pt, "RANK", rk);
  else
    Format(buf, sizeof(buf), "%t %u   %t - %t -\n ", "POINTS", pt, "RANK", "NONE_YET");

  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "->1. %t", "GOALS");
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "  %t %u   %t %u", "GOALS", gs, "MISSES", ms);
  DrawPanelText(pn, buf);

  new Float:gm_ratio = float(gs)/float(ms);
  Format(buf, sizeof(buf), "  - %t %.2f", "GM_RATIO", gm_ratio);
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "  %t %u   %t %u\n ", "ASSISTS", as, "OWN_GOALS", og);
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "->2. %t", "KEEPER");
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "  %t %u   %t %u", "SAVES", ks, "FAILS", kf);
  DrawPanelText(pn, buf);

  new Float:sf_ratio = float(ks)/float(kf);
  Format(buf, sizeof(buf), "  - %t %.2f\n ", "SF_RATIO", sf_ratio);
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "->3. %t", "BALL");
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "  %t\n  - %t %u   - %t %u", "PASSES", "GROUND", gp, "AIR", ap);
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "  %t %u   %t %u", "STEALS", sl, "LOSSES", ls);
  DrawPanelText(pn, buf);

  new Float:sl_ratio = float(sl)/float(ls);
  Format(buf, sizeof(buf), "  - %t %.2f\n ", "SL_RATIO", sl_ratio);
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "->4. %t", "ROUNDS");
  DrawPanelText(pn, buf);

  Format(buf, sizeof(buf), "  %t %u   %t %u\n ", "WON", rw, "LOST", rl);
  DrawPanelText(pn, buf);

  decl String:str_ftime[32];

  FriendlyTime(tp, str_ftime, sizeof(str_ftime));
  Format(buf, sizeof(buf), "%t %s", "TIME_PLAYED", str_ftime);

  DrawPanelText(pn, buf);

  if(fl & PLAYER_INGAME)
    Format(buf, sizeof(buf), "%t %t", "LAST_CONNECTED", "NOW");
  else
  {
    FriendlyTime(GetTime()-lc, str_ftime, sizeof(str_ftime));
    Format(buf, sizeof(buf), "%t %t", "LAST_CONNECTED", "N_AGO", str_ftime);
  }

  DrawPanelText(pn, buf);

  // respond to any key
  SetPanelKeys(pn, 0xFFFF);

  SendPanelToClient(pn, client, StatsReturn, PANEL_DISPLAYTIME);
  CloseHandle(pn);

  return;
}


public StatsReturn(Handle:panel,
                   MenuAction:action,
                   param1,
                   param2)
{
  return;
}
