// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs


PrintAnnouncementMsg(client)
{
  if(tracking_allowed)
  {
    if(client)
    // print to single client
      PrintToChat(client, "%s %t %t", ANNOUNCEMENT_PREFIX,
                                      "ANNOUNCEMENT_TRACKING", players_tracked,
                                      "ANNOUNCEMENT_HELP", help_cmd);
    else
    // broadcast to everyone
      for(new i = 1; i <= MaxClients; i++)
      if(IsClientInGame(i))
        PrintToChat(i, "%s %t %t", ANNOUNCEMENT_PREFIX,
                                   "ANNOUNCEMENT_TRACKING", players_tracked,
                                   "ANNOUNCEMENT_HELP", help_cmd);
  }
  else
  {
    if(client)
    // print to single client
      PrintToChat(client, "%s %t %t", ANNOUNCEMENT_PREFIX,
                                      "ANNOUNCEMENT_NOT_TRACKING",
                                      "ANNOUNCEMENT_HELP", help_cmd);
    else
    // broadcast to everyone
      for(new i = 1; i <= MaxClients; i++)
      if(IsClientInGame(i))
        PrintToChat(i, "%s %t %t", ANNOUNCEMENT_PREFIX,
                                   "ANNOUNCEMENT_NOT_TRACKING",
                                   "ANNOUNCEMENT_HELP", help_cmd);
  }

  return;
}


CheckTrackingAllowed(bool:round_start=false)
{
  if(!map_started)
    return;

  if(tracking_min_players < 1
  || GetTeamInitializedPlayerCount(CS_TEAM_T) < tracking_min_players
  || GetTeamInitializedPlayerCount(CS_TEAM_CT) < tracking_min_players)
  {
    if(!tracking_allowed)
    // tracking is already disallowed
      return;

    tracking_allowed = false;

    for(new i = 1; i <= MaxClients; i++)
    if(IsClientInGame(i))
      PrintToChat(i, "%s %t", ANNOUNCEMENT_PREFIX_PLAIN,
                              "TRACKING_DISABLED", tracking_min_players);
  }
  else
  if(round_start)
  // only re-enable tracking on round_start
  {
    if(tracking_allowed)
    // tracking is already allowed
      return;

    tracking_allowed = true;

    if(announcement_type == 1)
    // no round start messages allowed, so print this notice
      for(new i = 1; i <= MaxClients; i++)
      if(IsClientInGame(i))
        PrintToChat(i, "%s %t", ANNOUNCEMENT_PREFIX_PLAIN,
                                "TRACKING_ENABLED");
  }

  return;
}


GetTeamInitializedPlayerCount(team)
{
  new players = 0;

  for(new client = 1; client <= MaxClients; client++)
  if(p_init[client]
  && IsClientInGame(client)
  && GetClientTeam(client) == team)
    players++;

  return(players);
}


bool:DeadTimeBlock(client)
{
  // block all calls within defined deadtime
  static Float:time_nc[MAXPLAYERS+1];
  new Float:time_c = GetTickedTime();

  if(time_c < time_nc[client])
    return(true);

  time_nc[client] = time_c + PLUGIN_COMMAND_DEADTIME;
  return(false);
}


LateLoad()
{
  // hot load or reload of plugin
  // load all players from db
  ReadAllPlayersFromDatabase(h_statsdb);
  CheckTrackingAllowed();

  return;
}


TimeConnected(client)
{
  return(GetTime()-p_jointime[client]);
}


TotalTimePlayed(client)
{
  return(p_tp_onj[client]+TimeConnected(client));
}


DecayPoints(points, Float:decay)
{
  return(RoundToNearest(decay*(POINTS_DECAYTO-points)+points));
}


bool:IsPlayerRankable(client)
{
  // has to be matched with SQL_CALCULATE_RANKS in SQL defines
  if(p_props_abs[client][GROUND_PASSES]
    +p_props_abs[client][AIR_PASSES] >= min_passes_for_rank)
    return(true);

  return(false);
}


bool:FriendlyTime(time_s,
                  String:str_ftime[],
                  str_ftime_len,
                  bool:compact=true)
{
  decl String:days_pf[16];
  decl String:hrs_pf[16];
  decl String:mins_pf[16];
  decl String:secs_pf[16];

  if(compact)
  {
    days_pf = "d";
    hrs_pf = "h";
    mins_pf = "m";
    secs_pf = "s";
  }
  else
  {
    days_pf = " days";
    hrs_pf = " hours";
    mins_pf = " minutes";
    secs_pf = " seconds";
  }

  if(time_s < 0)
  {
    str_ftime[0] = '\0';
    return(false);
  }

  new days = time_s/86400;
  new hrs = (time_s/3600)%24;
  new mins = (time_s/60)%60;
  new secs = time_s%60;

  if(time_s < 60)
    Format(str_ftime, str_ftime_len,
           "%u%s",
           secs, secs_pf);
  else
  if(time_s < 3600)
    Format(str_ftime, str_ftime_len,
           "%u%s %u%s",
           mins, mins_pf, secs, secs_pf);
  else
  if(time_s < 86400)
    Format(str_ftime, str_ftime_len,
           "%u%s %u%s",
           hrs, hrs_pf, mins, mins_pf);
  else
    Format(str_ftime, str_ftime_len,
           "%u%s %u%s",
           days, days_pf, hrs, hrs_pf);

  return(true);
}


bool:MakeStringPrintable(String:str[],
                         str_len_max,
                         const String:empty[])
{
  // replaces sequences of \n and \t with space if
  // surrounded by non-space printable characters.
  // removes all other control characters, and if
  // the resulting string has zero-length or contains
  // only spaces, replaces it with the empty-string.

  new r = 0;
  new w = 0;

  new bool:modified = false;

  new bool:nonspace = false;
  new bool:addspace = false;

  if(str[0])
  do
  {
    if(str[r] < '\x20')
    {
      modified = true;

      if((str[r] == '\n'
      ||  str[r] == '\t')
      && w > 0
      && str[w-1] != '\x20')
        addspace = true;
    }
    else
    {
      if(str[r] != '\x20')
      {
        nonspace = true;

        if(addspace)
          str[w++] = '\x20';
      }

      addspace = false;
      str[w++] = str[r];
    }
  }
  while(str[++r]);
  str[w] = '\0';

  if(!nonspace)
  {
    modified = true;
    strcopy(str, str_len_max, empty);
  }

  return(modified);
}


bool:TruncateString(String:str[],
                    str_len,
                    truncate_to)
{
  if(str_len <= truncate_to)
    return(false);

  if(truncate_to < 3)
    truncate_to = 3;

  str[truncate_to-3] = '.';
  str[truncate_to-2] = '.';
  str[truncate_to-1] = '.';
  str[truncate_to] = '\0';

  return(true);
}


bool:IsStringInt(const String:str[],
                 bool:neg=false)
{
  // checks if string is an integer value
  // (can be negative if neg is true)
  // with at least one digit

  if(!str[0])
    return(false);

  new p = 0;

  if(str[p] == '-')
  {
    if(neg)
      p++;
    else
      return(false);
  }

  do
  {
    if(str[p] < '0'
    || str[p] > '9')
      return(false);
  }
  while(str[++p]);

  return(true);
}


StringToLower(String:str[])
{
  for(new c = 0; str[c]; c++)
  if(IsCharUpper(str[c]))
    str[c] |= (1<<5);

  return;
}
