// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs

static p_topoffs[MAXPLAYERS+1];


Top(client, offset)
{
  if(!p_init[client]
  || DeadTimeBlock(client))
    return;

  if(offset > TOPMENU_RANKS_PER_PAGE)
  {
    if(offset > rk_lowest)
      offset = rk_lowest;

    p_topoffs[client] = ((offset-1)
                        /TOPMENU_RANKS_PER_PAGE)
                        *TOPMENU_RANKS_PER_PAGE;
  }
  else
    p_topoffs[client] = 0;

  decl String:query[SQL_QUERY_MAXLEN];

  Format(query, sizeof(query), SQL_PLAYER_TOP_READ, TOPMENU_RANKS_PER_PAGE, p_topoffs[client]);
  SQL_TQuery(h_statsdb, SQL_OnPlayerTopRead, query, GetClientUserId(client), SQL_PLAYER_INTERACT_PRIORITY);

  return;
}


public SQL_OnPlayerTopRead(Handle:owner,
                           Handle:hndl,
                           const String:error[],
                           any:userid)
{
  if(hndl == INVALID_HANDLE)
  {
    LogError("SQL_OnPlayerTopRead failure: %s", error);
    return;
  }

  if(!SQL_HasResultSet(hndl))
  {
    LogError("SQL_OnPlayerTopRead failure: No result");
    return;
  }

  new client = GetClientOfUserId(userid);

  if(!client)
    return;

  if(!SQL_GetRowCount(hndl))
  {
    PrintToChat(client, "%t", "NO_PLAYERS_RANKED");
    return;
  }

  new Handle:pn = CreatePanel();
  SetGlobalTransTarget(client);

  // max length the panel can hold
  new len_avail = PANEL_MAX_LENGTH;

  decl nm_len[TOPMENU_RANKS_PER_PAGE];
  decl nm_len_trunc[TOPMENU_RANKS_PER_PAGE];

  decl String:rk_line[TOPMENU_RANKS_PER_PAGE][64];
  decl String:nm[TOPMENU_RANKS_PER_PAGE][MAX_NAME_LENGTH];

  decl String:pn_str_title[128];

  decl String:pn_str_back[16];
  decl String:pn_str_more[16];

  pn_str_back[0] = '\0';
  pn_str_more[0] = '\0';

  decl String:pn_str_exit[16];

  new rk_first = 0;
  new rk_last = 0;

  new nm_len_sum = 0;

  for(new i = 0; i < TOPMENU_RANKS_PER_PAGE; i++)
  // build rank line strings and count lengths
  {
    if(!SQL_FetchRow(hndl))
    // fill with blank line
    {
      len_avail -= Format(rk_line[i], sizeof(rk_line[]), " ")+1;
      nm_len[i] = (nm_len_trunc[i] = 0);
      continue;
    }

    new rk = SQL_FetchInt(hndl, 0);

    nm_len[i] = (nm_len_trunc[i] = SQL_FetchString(hndl, 1, nm[i], sizeof(nm[])));
    nm_len_sum += nm_len[i];

    new pt = SQL_FetchInt(hndl, 2);

    new Float:avg_ratio = (SQL_FetchFloat(hndl, 3)
                          +SQL_FetchFloat(hndl, 4)
                          +SQL_FetchFloat(hndl, 5))/3.0;

    // apply decay factor
    pt = DecayPoints(pt, SQL_FetchFloat(hndl, 6));

    if(i == 0)
      rk_first = rk;

    new fl = SQL_FetchInt(hndl, 7);

    if(nm[i][0])
    // normal rank line (-2 for %s which is replaced by name later)
      len_avail -= Format(rk_line[i], sizeof(rk_line[]), "%s%u %%s - %u  %.2f",
                          fl & PLAYER_INGAME? "->":"", rk, pt, avg_ratio)-2+1;
    else
    // no name -> rank reset -> we print msg
      len_avail -= Format(rk_line[i], sizeof(rk_line[]), "->%u - %t -",
                                                         rk, "PLAYERS_RANK_RESET")+1;
    rk_last = rk;
  }

  // subtract length of title + \n
  len_avail -= Format(pn_str_title, sizeof(pn_str_title), "->1. Forlix SoStats - %t\n \n%t",
                                                          "RANKS_N_TO_N", rk_first, rk_last,
                                                          "TOP_LABELS")+1;

  len_avail -= 4;
  // 2 blank lines (space + \n)

  // "back" button + \n or space + \n
  if(rk_first > TOPMENU_RANKS_PER_PAGE)
    len_avail -= Format(pn_str_back, sizeof(pn_str_back), "->8. %t", "BACK")+1;
  else
    len_avail -= 2;

  // "more" button + \n or space + \n
  if(rk_last < rk_lowest)
    len_avail -= Format(pn_str_more, sizeof(pn_str_more), "->9. %t", "MORE")+1;
  else
    len_avail -= 2;

  len_avail -= Format(pn_str_exit, sizeof(pn_str_exit), "0. %t", "EXIT");
  // exit without \n (we let it be truncated)

  while(nm_len_sum > len_avail)
  // keep on truncating until names fit
  {
    new longest = 0;

    for(new i = 1; i < TOPMENU_RANKS_PER_PAGE; i++)
    // find the longest name
    if(nm_len_trunc[i] > nm_len_trunc[longest])
      longest = i;

    // cut off 1 char
    nm_len_trunc[longest]--;
    nm_len_sum--;
  }

  // begin drawing the actual panel
  DrawPanelText(pn, pn_str_title);

  for(new i = 0; i < TOPMENU_RANKS_PER_PAGE; i++)
  // print TOPMENU_RANKS_PER_PAGE rows
  {
    decl String:buf[sizeof(rk_line[])+sizeof(nm[])];

    if(nm[i][0])
    {
      TruncateString(nm[i], nm_len[i], nm_len_trunc[i]);
      Format(buf, sizeof(buf), rk_line[i], nm[i]);
    }
    else
      Format(buf, sizeof(buf), rk_line[i]);

    DrawPanelText(pn, buf);
  }

  DrawPanelText(pn, " ");

  new pn_keys;

  if(pn_str_back[0])
  // add "back" button (8)
  {
    pn_keys |= (1<<8-1);
    DrawPanelText(pn, pn_str_back);
  }
  else
  // empty line
    DrawPanelText(pn, " ");

  if(pn_str_more[0])
  // add "more" button (9)
  {
    pn_keys |= (1<<9-1);
    DrawPanelText(pn, pn_str_more);
  }
  else
  // empty line
    DrawPanelText(pn, " ");

  DrawPanelText(pn, " ");

  pn_keys |= (1<<10-1);
  DrawPanelText(pn, pn_str_exit);

  SetPanelKeys(pn, pn_keys);

  SendPanelToClient(pn, client, TopReturn, PANEL_DISPLAYTIME);
  CloseHandle(pn);

  return;
}


public TopReturn(Handle:panel,
                 MenuAction:action,
                 param1,
                 param2)
{
  if(action != MenuAction_Select
  || param2 == 10)
    return;

  new client = param1;

  if(param2 == 8)
  // player selected "back"
    p_topoffs[client] -= TOPMENU_RANKS_PER_PAGE;
  else
  if(param2 == 9)
  // player selected "more"
    p_topoffs[client] += TOPMENU_RANKS_PER_PAGE;

  decl String:query[SQL_QUERY_MAXLEN];

  Format(query, sizeof(query), SQL_PLAYER_TOP_READ, TOPMENU_RANKS_PER_PAGE, p_topoffs[client]);
  SQL_TQuery(h_statsdb, SQL_OnPlayerTopRead, query, GetClientUserId(client), SQL_PLAYER_INTERACT_PRIORITY);

  return;
}
