// Forlix SoStats
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2009-2011 Dominik Friedrichs


public ForlixSoScore_OnBallPassed(passedfrom_userid,
                                  passedto_userid,
                                  team,
                                  Float:distance_normalized,
                                  Float:speed_averaged,
                                  bool:air_shot)
{
  new passedfrom = GetClientOfUserId(passedfrom_userid);
  new passedto = GetClientOfUserId(passedto_userid);

  if(!passedfrom
  || !passedto
  || !p_init[passedfrom]
  || !p_init[passedto]
  || !tracking_allowed)
    return;

  new Float:pt = distance_normalized*10.0;
  // 10 pt for pass from goal to goal

  if(speed_averaged > 2000)
    speed_averaged = 2000.0;

  pt *= (speed_averaged/1000)+0.25;
  // factor 1 at 750 speed, factor 2 at 1750 speed

  if(air_shot)
  // 75% bonus for air passes
    TrackProperty(passedfrom, AIR_PASSES, pt*1.75);
  else
    TrackProperty(passedfrom, GROUND_PASSES, pt);

  return;
}


public ForlixSoScore_OnBallTurnedOver(stealer_userid,
                                      stealer_team,
                                      loser_userid,
                                      loser_team,
                                      Float:distance_normalized,
                                      Float:speed_averaged,
                                      bool:air_shot)
{
  new stealer = GetClientOfUserId(stealer_userid);
  new loser = GetClientOfUserId(loser_userid);

  if(!stealer
  || !loser
  || !p_init[stealer]
  || !p_init[loser]
  || !tracking_allowed)
    return;

  new Float:pt = 10-(distance_normalized*18.0);
  // distance 0 = 10 pt
  // distance >= 0.5 = 1 pt
  if(pt < 1)
    pt = 1.0;

  if(speed_averaged > 2000)
    speed_averaged = 2000.0;

  pt *= (speed_averaged/1000)+0.25;
  // factor 1 at 750 speed, factor 2 at 1750 speed

  if(air_shot)
  // 75% bonus for air steals
    pt *= 1.75;

  new Float:sf = SkillFactor(p_pt_abs[loser], p_pt_abs[stealer]);

  if(IsPlayerRankable(stealer))
    TrackProperty(loser, LOSSES, -1*pt*sf);
  else
    TrackProperty(loser, LOSSES, -1*pt);

  TrackProperty(stealer, STEALS, pt*sf);
  return;
}


public ForlixSoScore_OnGoalScored(winning_team,
                                  losing_team,
                                  shooter_userid,
                                  assistant_userid,
                                  keeper_userids[],
                                  bool:owngoal,
                                  Float:distance_normalized,
                                  Float:speed_averaged,
                                  bool:air_shot,
                                  Float:time_into_round)
{
  new shooter = GetClientOfUserId(shooter_userid);
  new assistant = GetClientOfUserId(assistant_userid);

  if(!shooter
  || !p_init[shooter]
  || !tracking_allowed)
    return;

  new Float:dist_fact = distance_normalized+0.75;
  // factor 1 from (1/4*goal-to-goal dist) to goal, factor 1.75 from goal-to-goal dist

  new Float:time_fact = (time_into_round/240)+0.5;
  // factor 1 after 2 minutes, factor 3 after 10 minutes

  new Float:speed_fact = 1.5-(speed_averaged/2000);
  if(speed_fact < 0.8)
    speed_fact = 0.8;
  // factor 1.5 at 0 speed, factor 1 at 1000 speed, factor 0.8 at 1400 speed and higher

  new Float:fact = dist_fact*time_fact*speed_fact;

  if(air_shot)
  // 75% bonus for air shots
    fact *= 1.75;

  new pcnt_loser = GetTeamInitializedPlayerCount(losing_team);
  new pcnt_winner = GetTeamInitializedPlayerCount(winning_team);

  fact *= pcnt_loser/6.0;
  // factor 1 at 6 player loser team

  // there will always be at least 1 player per team
  new tpavg_loser = TeamPointSum(losing_team)/pcnt_loser;
  new tpavg_winner = TeamPointSum(winning_team)/pcnt_winner;

  if(!owngoal)
  // normal goal
  {
    // everyone in the team loses or gains points
    for(new i = 1; i <= MaxClients; i++)
    if(p_init[i]
    && IsClientInGame(i))
    {
      new team = GetClientTeam(i);

      if(team == winning_team)
        TrackProperty(i, ROUNDS_WON, 10*fact*SkillFactor(tpavg_loser, p_pt_abs[i]));
      else
      if(team == losing_team)
        TrackProperty(i, ROUNDS_LOST, -10*fact*SkillFactor(p_pt_abs[i], tpavg_winner));
    }

    // shooters and assistants get extra points on top
    if(assistant
    && p_init[assistant])
    // more points for shooter
    // if goal was assisted
    {
      TrackProperty(shooter, GOALS, 20*fact*SkillFactor(tpavg_loser, p_pt_abs[shooter]));
      TrackProperty(assistant, ASSISTS, 20*fact*SkillFactor(tpavg_loser, p_pt_abs[assistant]));
    }
    else
    // no assistant
      TrackProperty(shooter, GOALS, 10*fact*SkillFactor(tpavg_loser, p_pt_abs[shooter]));

    for(new i = 0; keeper_userids[i] > 0; i++)
    {
      new keeper = GetClientOfUserId(keeper_userids[i]);

      if(keeper
      && p_init[keeper])
        TrackProperty(keeper, KEEPER_FAILS, -20*fact*SkillFactor(p_pt_abs[keeper], p_pt_abs[shooter]));
    }
  }
  else
  // own goal
  {
    for(new i = 1; i <= MaxClients; i++)
    if(p_init[i]
    && IsClientInGame(i))
    {
      new team = GetClientTeam(i);

      if(team == winning_team)
        TrackProperty(i, ROUNDS_WON, 0.0);
      else
      if(team == losing_team)
        TrackProperty(i, ROUNDS_LOST, 0.0);
    }

    for(new i = 0; keeper_userids[i] > 0; i++)
    {
      new keeper = GetClientOfUserId(keeper_userids[i]);

      if(keeper
      && p_init[keeper])
        TrackProperty(keeper, KEEPER_FAILS, 0.0);
    }

    TrackProperty(shooter, OWN_GOALS, -50*fact*SkillFactor(p_pt_abs[shooter], tpavg_loser));
  }

  return;
}


public ForlixSoScore_OnGoalMissed(shooter_userid,
                                  shooter_team,
                                  missed_goal,
                                  Float:distance_normalized,
                                  Float:speed_averaged,
                                  bool:air_shot)
{
  new shooter = GetClientOfUserId(shooter_userid);

  if(!shooter
  || !p_init[shooter]
  || !tracking_allowed)
    return;

  if(shooter_team == missed_goal)
  // dont count misses on own goal
    return;

  TrackProperty(shooter, MISSES, 0.0);
  return;
}


public ForlixSoScore_OnGoalSaved(keeper_userid,
                                 keeper_team,
                                 shooter_userid,
                                 shooter_team,
                                 Float:distance_normalized,
                                 Float:speed_averaged,
                                 bool:air_shot)
{
  new keeper = GetClientOfUserId(keeper_userid);
  new shooter = GetClientOfUserId(shooter_userid);

  if(!keeper
  || !shooter
  || !p_init[keeper]
  || !p_init[shooter]
  || !tracking_allowed)
    return;

  if(keeper_team == shooter_team)
  // dont count saves of friendly shots
    return;

  new Float:dist_fact = 1/(distance_normalized+0.75);
  // factor 1 from (1/4*goal-to-goal dist) to goal, factor 0.6 from goal-to-goal dist

  new Float:speed_fact = (speed_averaged/1000)+0.25;
  if(speed_fact > 2)
    speed_fact = 2.0;
  // factor 1 at 750 speed, factor 2 at 1750 speed and higher

  new Float:fact = dist_fact*speed_fact;

  if(air_shot)
  // 75% bonus for air shots
    fact *= 1.75;

  TrackProperty(keeper, KEEPER_SAVES, 5*fact*SkillFactor(p_pt_abs[shooter], p_pt_abs[keeper]));
  return;
}


Float:SkillFactor(point_source,
                  point_target)
{
  new Float:ratio = float(point_source)
                   /float(point_target);

  if(ratio > 3)
    ratio = 3.0;
  else
  if(ratio < 1)
    ratio = 1.0;

  return(ratio);
}


TrackProperty(client,
              prop,
              Float:pt_delta)
{
  new pt_rounded = RoundToNearest(pt_delta);

  p_props_abs[client][prop]++;
  new pt = p_pt_abs[client];

  if(pt_rounded)
  {
    pt += pt_rounded;

    if(pt < POINTS_MINIMUM)
      pt = POINTS_MINIMUM;

    p_pt_abs[client] = pt;
  }

  if(p_fl_abs[client] & PLAYER_TRACKMSGS)
    PrintToChat(client, "\x03%s%i\x01 %t \x03%t",
                        pt_rounded > 0? "+":"", pt_rounded, "FOR", prop_desc[prop]);

  HintToPlayer(client, prop);
  return;
}


TeamPointSum(team)
{
  new pt_sum = 0;

  for(new client = 1; client <= MaxClients; client++)
  if(p_init[client]
  && IsClientInGame(client)
  && GetClientTeam(client) == team)
    pt_sum += p_pt_abs[client];

  return(pt_sum);
}
